using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;

using jade.core;
using jade.core.behaviours;

using jade.lang.acl;
using java.lang;

using jade.domain;
using jade.domain.FIPAAgentManagement;
using jade.domain.DFGUIManagement;

namespace MASTask2
{
    class BoneServer : Agent
    {
        public override void setup()
        {
            //here will be parallel behaviour - dealing with players:
            addBehaviour(new GameManageBehaviour(this));
 
            //Registration:
            //here will be sequence behaviour - registration:
            DFAgentDescription dFDescription = new DFAgentDescription();
            ServiceDescription sDescription = new ServiceDescription();
            sDescription.setType("GamerReg");
            dFDescription.addServices(sDescription);
            SearchConstraints searchConstraints = new SearchConstraints();
            searchConstraints.setMaxResults(new Long(-1));

            send(DFService.createSubscriptionMessage(this, getDefaultDF(), dFDescription, searchConstraints));

        }
    }
    class GameManageBehaviour : Behaviour
    {
        private bool finished = false;
        private bool wait = false;
        private string firstPlayerName;
        private GamerList gamerList = new GamerList();
        private int incomingCount = 0;

        public GameManageBehaviour(Agent agent) : base(agent)
        {

        }
        public override void action()
        {
            ACLMessage message = myAgent.receive(MessageTemplate.MatchSender(myAgent.getDefaultDF()));

            if (message != null)
            {
                try
                {
                    DFAgentDescription[] dFAgentDescription = DFService.decodeNotification(message.getContent());
                    for (int i = 0; i < dFAgentDescription.Length; i++)
                    {
                        if (wait)
                        {
                            wait = false;
                            Pair pair = new Pair(firstPlayerName, dFAgentDescription[i].getName().getLocalName());
                            gamerList.addPair(pair);
                            incomingCount++;
                            say(firstPlayerName + " and " + dFAgentDescription[i].getName().getLocalName() + " are paired in game");
                            myAgent.addBehaviour(new GameAdministrateBehaviour(myAgent, firstPlayerName, dFAgentDescription[i].getName().getLocalName()));
                            continue;
                        }
                        else
                        {
                            wait = true;
                            firstPlayerName = dFAgentDescription[i].getName().getLocalName();
                            incomingCount++; 
                            continue;
                         }

                    }
             /*         if (incomingCount == 5)                   
                          myAgent.addBehaviour(new GameAdministrateBehaviour(myAgent, gamerList));
              */  
                }
                catch (java.lang.Exception e)
                {
                    e.printStackTrace();
                }
            }

            block();

        }
        private void say(string message)
        {
            System.Console.WriteLine("[" + myAgent.getLocalName() + "]: " + message + "\n");
        }
        public override bool done()
        {
            return finished;
        }

    }
    class GamerList
    {
        private ArrayList gamerList = new ArrayList(); 

        public GamerList()
        {
            
        }
        public void addPair(Pair pair)
        {
            gamerList.Add(pair);
        }
        public Pair getPair(int index)
        {
            return (Pair)gamerList[index];
        }
        public int getCount()
        {
            return gamerList.Count;
        }
        public void removePair(int index)
        {
            gamerList.RemoveAt(index);
        }

    }
    class Pair
    {
        public string firstPlayerName, secondPlayerName;

        public Pair(string firstPlayer, string secondPlayer)
        {
            firstPlayerName = firstPlayer;
            secondPlayerName = secondPlayer;
        }
    }


    // it will be subbehaviour of parallel behaviour:
    class GameAdministrateBehaviour : Behaviour
    {
        private bool finished = false;
        private int state = 0;
        private string gamerOne, gamerTwo;
        private int srvcCount = 0;
        private int swap = 0;
        private int firstShot, secondShot;
        private AID playerOneAID, playerTwoAID;      
        private int current = 0;
        private int roundNumber = 10;
        private Random random = new Random(DateTime.Now.Millisecond);

       

        public GameAdministrateBehaviour(Agent agent, string playerOne, string playerTwo) : base(agent)
        {        
       //     gamerList = gamerListT;
            gamerOne = playerOne;
            gamerTwo = playerTwo;
      
        }
        public override void action()
        {
/*            if (gamerList.getCount() != 0)
            {
                gamerOne = gamerList.getPair(current).firstPlayerName;
                gamerTwo = gamerList.getPair(current).secondPlayerName;
            }
*/  
            switch (state)
            {
                case 0:
                    {                                          
                            ACLMessage ready = myAgent.receive(MessageTemplate.or
                                (MessageTemplate.MatchConversationId(gamerOne),
                                MessageTemplate.MatchConversationId(gamerTwo)));
                            if (ready != null)
                            {
                                ACLMessage responseOn = new ACLMessage(ACLMessage.INFORM);

                                if (ready.getSender().getLocalName() == gamerOne)
                                {
                                    responseOn.setContent(gamerTwo);
                                    playerOneAID = new AID(ready.getSender().getLocalName(), AID.ISLOCALNAME);
                                    block();
                                }
                                else
                                {
                                    responseOn.setContent(gamerOne);
                                    playerTwoAID = new AID(ready.getSender().getLocalName(), AID.ISLOCALNAME);
                                    block();
                                }

                                srvcCount++;
                                responseOn.addReceiver(ready.getSender());
                                responseOn.setConversationId(ready.getSender().getLocalName());
                                myAgent.send(responseOn);
                                if (srvcCount == 2)
                                {
                                    srvcCount = 0;
                                    state = 1;
                                    block();
                                }
                                break;
                            }
                      
                        block();
                        break;
                    }
                case 1:
                    {
             
                            ACLMessage incoming = myAgent.receive(MessageTemplate.or
                                (MessageTemplate.MatchConversationId(gamerOne),
                                MessageTemplate.MatchConversationId(gamerTwo)));
                            if (incoming != null)
                            {

                                
                                
                                int shot = random.Next(1, 3) + random.Next(1, 3);
                                say(incoming.getSender().getLocalName() + " has " + shot + "points");
                                ACLMessage response = new ACLMessage(ACLMessage.INFORM);
                                response.setContent(shot.ToString());
                                response.addReceiver(new AID(incoming.getSender().getLocalName(), AID.ISLOCALNAME));
                                myAgent.send(response);
                                srvcCount++;
                                if (swap == 0)
                                {
                                    firstShot = shot;
                                    swap = 1;
                                }
                                else if (swap == 1)
                                {
                                    secondShot = shot;
                                    swap = 0;
                                }
                                if (srvcCount == 2)
                                {
                                    ACLMessage orderToFirst = new ACLMessage(ACLMessage.INFORM);
                                    ACLMessage orderToSecond = new ACLMessage(ACLMessage.INFORM);
                                    srvcCount = 0;
                                    int codedPoints = firstShot * 100 + secondShot;
                                    orderToFirst.setContent(codedPoints.ToString());
                                    codedPoints = secondShot * 100 + firstShot;
                                    orderToSecond.setContent(codedPoints.ToString());
                                    orderToFirst.addReceiver(playerOneAID);
                                    orderToSecond.addReceiver(playerTwoAID);
                                    myAgent.send(orderToFirst);
                                    myAgent.send(orderToSecond);
                                    if (firstShot != secondShot)
                                    {
                                        current++;
                                   
                                        state = 0;
                                        block();
                                    }
                                    roundNumber--;
                                    if (roundNumber <= 0)
                                    {
                                        finished = true;
                                        break;
                                    }
                                }                              
                                block(0);
                            }                                              
                        block();
                        break;
                    }
                   
               
            }
        }
        public override bool done()
        {
            return finished;
        }
        private void say(string message)
        {
            System.Console.WriteLine("[" + myAgent.getLocalName() + "]: " + message + "\n");
        }

    }
}
