using System;
using System.Collections.Generic;
using System.Text;

using jade.core;
using jade.core.behaviours;

using jade.domain;
using jade.domain.FIPAAgentManagement;
using jade.domain.DFGUIManagement;

using jade.lang.acl;

namespace MASTask2
{
    class BoneGamer : Agent
    {
        public override void setup()
        {
            say("arrival");

            //Here will be subbehaviour - playing:
            addBehaviour(new WaitingBehaviour(this));
            
            //Here will be suquential behaviour : 
            //  here will be subbehaviour - registration:
            DFAgentDescription dFDescription = new DFAgentDescription();
            dFDescription.setName(this.getAID());
            ServiceDescription serviceDescription = new ServiceDescription();
            serviceDescription.setType("GamerReg");
            serviceDescription.setName(this.getLocalName());
            dFDescription.addServices(serviceDescription);
            try
            {
                DFService.register(this, dFDescription);
            }
            catch (java.lang.Exception exc)
            {
                exc.printStackTrace();
                return;
            }

        }
        public override void takeDown()
        {
            try 
            {
                DFService.deregister(this); 
            }
            catch (FIPAException e)
            {
                e.printStackTrace();
            }
            base.takeDown();
        }
        private void say(string message)
        {
            System.Console.WriteLine("[" + getLocalName() + "]: " + message + "\n");            
        }
    }
    class WaitingBehaviour : Behaviour
    {
        private bool finished = false;

        public WaitingBehaviour(Agent agent) : base(agent)
        {
            say("waiting state");

            ACLMessage greet = new ACLMessage(ACLMessage.INFORM);
            greet.setConversationId(myAgent.getLocalName());
            greet.setContent("ready");
            greet.addReceiver(new AID("BoneServer", AID.ISLOCALNAME));
            myAgent.send(greet);
        }
        public override void  action()
        {
            ACLMessage message = myAgent.receive();

            //waiting for another agent to play with:
            if (message != null)
            {
                say("paired with " + message.getContent());
                myAgent.addBehaviour(new PlayingBehaviour(myAgent, message.getContent()));
                finished = true;              
            }
            block();
        }
        public override bool done()
        {
            return finished;
        }
        private void say(string message)
        {
            System.Console.WriteLine("[" + myAgent.getLocalName() + "]: " + message + "\n");
        }

    }
    class PlayingBehaviour : Behaviour
    {
        private bool finished = false;
        private string other;
        private AID otherAID;
   

        private int state = 0;
        /*  STATES:
         *  0 - wait for orders for doing next turn
         *  1 - obtaining results
         * 
        */
        public PlayingBehaviour(Agent agent, string theOther) : base(agent)
        {
            say("confirmation's been obtained");
            other = theOther;
            otherAID = new AID(other, AID.ISLOCALNAME);
         

            ACLMessage waitForOrders = new ACLMessage(ACLMessage.INFORM);
            waitForOrders.setConversationId(myAgent.getLocalName());
            waitForOrders.setContent("WaitForOrders");
            waitForOrders.addReceiver(new AID("BoneServer", AID.ISLOCALNAME));
            myAgent.send(waitForOrders);
        }
        public override void action()
        {
            switch(state)
            {
                case 0:
                    {
                        ACLMessage incoming = myAgent.blockingReceive();
                        if (incoming != null)
                        {
                            ACLMessage turn = new ACLMessage(ACLMessage.INFORM);
                            turn.setConversationId(myAgent.getLocalName());
                            turn.setContent("throw");
                            turn.addReceiver(new AID("BoneServer", AID.ISLOCALNAME));
                            myAgent.send(turn);
                            state = 1;
                          
                        }
                        block();
                        break;
                    }
                case 1:
                    {
                        ACLMessage incoming = myAgent.receive();
//                        ACLMessage incoming = myAgent.blockingReceive();
                        if (incoming != null)
                        {
                            int result = Int16.Parse(incoming.getContent());
                            int myResult = (int)(result / 100);
                            int otherResult = result - myResult * 100;

                            if (myResult < otherResult)
                            {
                                say("loser");
                                finished = true;
                            }
                            if (myResult > otherResult)
                            {
                                say("winner");
                                
                                finished = true;
                            }
                            if (myResult == otherResult)
                            {
                                say("draw");
                                state = 0;
                                ACLMessage again = new ACLMessage(ACLMessage.INFORM);
                                again.setContent("again");
                                again.addReceiver(new AID("BoneServer", AID.ISLOCALNAME));
                                myAgent.send(again);
                            }
                        }
                        block();
                        break;
                    }
            }
        }
        public override bool done()
        {
            return finished;
        }
        private void say(string message)
        {
            System.Console.WriteLine("[" + myAgent.getLocalName() + "]: " + message + "\n");
        }
 
    }
}
